@@ -78,16 +78,16 @@ public class ClientConfig implements Config {
     public URL REMOTEURL_IMAGESERVER = null;
 
     /**
-     * the maximum size in MegaBytes for documents to be converted;
-     * -1 for no upper limit
+     * the maximum size in Bytes for documents to be converted;
+     * -1 for no upper limit; default is 128MB
      */
-    public long MAX_DOCUMENT_SOURCEFILESIZE_MB = 128;
+    public long MAX_DOCUMENT_SOURCEFILESIZE = 134217728;
 
     /**
      * the maximum size in MegaBytes for documents to be converted;
-     * -1 for no upper limit
+     * -1 for no upper limit; default is 32MB
      */
-    public long MAX_IMAGE_SOURCEFILESIZE_MB= 32;
+    public long MAX_IMAGE_SOURCEFILESIZE= 33554432;
 
     public boolean PREVIEW_ENABLE_CACHE_LOOKUP = true;
 
