@@ -61,9 +61,7 @@ import org.apache.commons.lang.StringUtils;
 import org.slf4j.Logger;
 import com.openexchange.ajax.fileholder.IFileHolder;
 import com.openexchange.documentconverter.AsyncExecutor;
-import com.openexchange.documentconverter.Feature;
 import com.openexchange.documentconverter.IDocumentConverter;
-import com.openexchange.documentconverter.IFeature;
 import com.openexchange.documentconverter.IImageTransformer;
 import com.openexchange.documentconverter.ILogProvider;
 import com.openexchange.documentconverter.ImageServerTransformations;
@@ -84,7 +82,7 @@ import com.openexchange.server.ServiceLookup;
  * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
  * @since v7.8.0
  */
-public class ClientManager extends ManagerBasics implements IFeature, IDocumentConverter, IImageTransformer, ImageServerTransformationService {
+public class ClientManager extends ManagerBasics implements IDocumentConverter, IImageTransformer, ImageServerTransformationService {
 
 
     /**
@@ -102,14 +100,18 @@ public class ClientManager extends ManagerBasics implements IFeature, IDocumentC
 
         m_asyncExecutor = new AsyncExecutor(this, this, ASYNC_CONVERTER_THREADCOUNT);
 
-        // create remoteValidators for each configured remote server type
-        if ((null != clientConfig.REMOTEURL_DOCUMENTCONVERTER) && (clientConfig.REMOTEURL_DOCUMENTCONVERTER.toString().length() > 0)) {
-            m_remoteDocumentConverterValidator = new RemoteValidator(ServerType.DOCUMENTCONVERTER, clientConfig.REMOTEURL_DOCUMENTCONVERTER);
+     // create remoteValidators for each configured remote server type
+        if ((null != m_clientConfig.REMOTEURL_DOCUMENTCONVERTER) && (m_clientConfig.REMOTEURL_DOCUMENTCONVERTER.toString().length() > 0)) {
+            m_remoteDocumentConverterValidator = new RemoteValidator(ServerType.DOCUMENTCONVERTER, m_clientConfig.REMOTEURL_DOCUMENTCONVERTER);
+        } else if (isLogError()) {
+            logError("Documentconverter remoteDocumentConverterUrl property is empty or not set at all. Client is not able to perform conversions until a valid Url is set!");
         }
 
         // create remoteValidators for each configured remote server type
-        if ((null != clientConfig.REMOTEURL_IMAGESERVER) && (clientConfig.REMOTEURL_IMAGESERVER.toString().length() > 0)) {
-            m_remoteImageServerValidator = new RemoteValidator(ServerType.IMAGESERVER, clientConfig.REMOTEURL_IMAGESERVER);
+        if ((null != m_clientConfig.REMOTEURL_IMAGESERVER) && (m_clientConfig.REMOTEURL_IMAGESERVER.toString().length() > 0)) {
+            m_remoteImageServerValidator = new RemoteValidator(ServerType.IMAGESERVER, m_clientConfig.REMOTEURL_IMAGESERVER);
+        } else if (isLogError()) {
+            logError("Documentconverter remoteImageServerUrl property is empty or not set at all. Client is not able to perform image transformations until a valid Url is set!");
         }
     }
 
@@ -126,16 +128,6 @@ public class ClientManager extends ManagerBasics implements IFeature, IDocumentC
         return m_clientManager;
     }
 
-
-    /* (non-Javadoc)
-     * @see com.openexchange.documentconverter.IClientManager#hasFeature(com.openexchange.documentconverter.Feature, java.lang.Object[])
-     */
-    @Override
-    public boolean hasFeature(Feature feature, Object... params) {
-        // TODO (KA)
-        return true;
-    }
-
     /* (non-Javadoc)
      * @see com.openexchange.documentconverter.IClientManager#convert(java.lang.String, java.util.HashMap, java.util.HashMap)
      */
@@ -177,7 +169,13 @@ public class ClientManager extends ManagerBasics implements IFeature, IDocumentC
         TransformedImage ret = null;
 
         if ((null != imageInputStm) && !StringUtils.isEmpty(formatName) && (null != userData) && (userData instanceof ImageServerTransformations)) {
-            // TODO (KA)
+            final ImageTransformations imageTransformations = m_services.getOptionalService(ImageTransformations.class);
+
+            if (null != imageTransformations) {
+                final ImageServerTransformations imageServerTransformations = (ImageServerTransformations) userData;
+
+                // TODO (KA): impl.
+            }
         }
 
         return ret;
