@@ -122,6 +122,29 @@ class ManagedClientJob {
         }
     }
 
+    /**
+     * @param jobType
+     * @param jobProperties
+     * @param resultProperties
+     * @return
+     */
+    boolean queryAvailability(String jobType, HashMap<String, Object> jobProperties, HashMap<String, Object> resultProperties) {
+        boolean ret = false;
+
+        if (checkJobPreconditions(jobProperties, resultProperties, 1)) {
+            if (prepareRemoteCall(jobProperties, RCALL_QUERY_AVAILABILITY)) {
+                jobProperties.put(Properties.PROP_JOBTYPE, jobType);
+
+                doRemoteCall(jobProperties, resultProperties);
+
+                ret = resultProperties.containsKey(Properties.PROP_RESULT_ERROR_CODE) &&
+                        (JobError.NONE == JobError.fromErrorCode(((Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE)).intValue()));
+            }
+        }
+
+        return ret;
+    }
+
     /*
      * (non-Javadoc)
      * @see com.openexchange.documentconverter.impl.ManagedJob#convert(java.lang.String, java.util.HashMap, java.util.HashMap)
@@ -131,7 +154,9 @@ class ManagedClientJob {
         final Boolean asyncObject = (Boolean) jobProperties.get(Properties.PROP_ASYNC);
 
         if ((null != asyncObject) && asyncObject.booleanValue()) {
-            m_clientManager.triggerAsyncConvert(jobType, jobProperties, resultProperties);
+            if (checkJobInputFileSize(jobProperties, resultProperties)) {
+                m_clientManager.triggerAsyncConvert(jobType, jobProperties, resultProperties);
+            }
         } else if (checkJobPreconditions(jobProperties, resultProperties, 1)) {
             if (prepareRemoteCall(jobProperties, RCALL_CONVERT)) {
                 jobProperties.put(Properties.PROP_JOBTYPE, jobType);
@@ -218,7 +243,7 @@ class ManagedClientJob {
      * @param jobProperties
      * @return
      */
-    static protected byte[] getInputBuffer(HashMap<String, Object> jobProperties) {
+    static protected byte[] getInputBuffer(final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties) {
         @SuppressWarnings("resource")
         InputStream inputStm = null;
         byte[] ret = null;
@@ -239,6 +264,11 @@ class ManagedClientJob {
             IOUtils.closeQuietly(inputStm);
         }
 
+        // check, if length of byte array is larger than specified config source file size
+        if ((null != ret) && (setSourceFileSizeError(ret.length, jobProperties, resultProperties))) {
+            ret = null;
+        }
+
         return ret;
     }
 
@@ -246,25 +276,22 @@ class ManagedClientJob {
 
     /**
      * @param jobProperties
+     */
+    /**
+     * @param jobProperties
+     * @param resultProperties
+     * @param minRemoteAPIVersion
      * @return true, if job processing can proceed
      */
-    protected boolean checkJobPreconditions(HashMap<String, Object> jobProperties, HashMap<String, Object> resultProperties, int minRemoteAPIVersion) {
+    protected boolean checkJobPreconditions(final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties, int minRemoteAPIVersion) {
         boolean ret = false;
 
         if (null != jobProperties) {
             // job is valid by default, if job properties exists at all
-            ret = true;
-
-            // check max source file size, if source file is given
-            final File inputFile = (File) jobProperties.get(Properties.PROP_INPUT_FILE);
-
-            if (null != inputFile) {
-                final long maxSourceFileSize = ClientManager.getClientConfig().MAX_DOCUMENT_SOURCEFILESIZE_MB * 1024 * 1024;
-
-                if (!(ret = (maxSourceFileSize < 0) || (inputFile.length() <= maxSourceFileSize))) {
-                    resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.MAX_SOURCESIZE.errorCode()));
-                }
-            }
+            // and source file size fits into the given constraints
+            ret = checkJobInputFileSize(jobProperties, resultProperties);
+        } else {
+            resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.GENERAL.errorCode()));
         }
 
         if (ret && (null != m_remoteUrl)) {
@@ -274,6 +301,62 @@ class ManagedClientJob {
         return ret;
     }
 
+    /**
+     * @param jobProperties
+     * @param resultProperties
+     * @return true, if source file size is unlimited or below/equal
+     *  to the given maximum source file size
+     */
+    protected static boolean checkJobInputFileSize(final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties) {
+        boolean ret = false;
+
+        if (null != jobProperties) {
+            // check max source file size, if source file is given
+            final File inputFile = (File) jobProperties.get(Properties.PROP_INPUT_FILE);
+
+            if (null != inputFile) {
+                // check maxSourceFile
+                ret = !setSourceFileSizeError(inputFile.length(), jobProperties, resultProperties);
+            } else {
+                // job is valid by default, if job properties exists
+                // at all and source file is given as input stream
+                ret = true;
+            }
+        } else {
+            resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.GENERAL.errorCode()));
+        }
+
+        return ret;
+    }
+
+    /**
+     * @param sourceSize
+     * @param maxSize
+     * @param resultProperties
+     * @return true, if source size is larger than max size
+     */
+    protected static boolean setSourceFileSizeError(long sourceSize, final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties) {
+        final long maxSize = ClientManager.getClientConfig().MAX_DOCUMENT_SOURCEFILESIZE;
+        boolean ret = false;
+
+        if (sourceSize > maxSize) {
+            ret = true;
+
+            if (null != resultProperties) {
+                resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.MAX_SOURCESIZE.errorCode()));
+
+                if (ClientManager.isLogError()) {
+                    ClientManager.logError("Documentconverter detected source file size larger than configured size",
+                        jobProperties,
+                        new LogData("sourcesize", (new Long(sourceSize)).toString()),
+                        new LogData("maxsize", (new Long(maxSize)).toString()));
+                }
+            }
+        }
+
+        return ret;
+    }
+
     /**
      * @param jobProperties
      * @param method
@@ -324,25 +407,25 @@ class ManagedClientJob {
     private Object doRemoteCall(final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties) {
         Object ret = null;
         final boolean isMultipartRequest = (jobProperties.containsKey(Properties.PROP_INPUT_FILE) || jobProperties.containsKey(Properties.PROP_INPUT_STREAM));
-        final String remoteParams = isMultipartRequest ?
-            getRemoteParams(jobProperties, false, "returntype=binary") :
-                getRemoteParams(jobProperties, true);
+        final Boolean asyncJob = (Boolean) jobProperties.get(Properties.PROP_ASYNC);
+        final Boolean remoteAsyncJob = (Boolean) jobProperties.get(Properties.PROP_REMOTE_ASYNC);
+        final Boolean cacheOnlyJob = (Boolean) jobProperties.get(Properties.PROP_CACHE_ONLY);
+        final String remoteCacheHash = (String) jobProperties.get(Properties.PROP_REMOTE_CACHE_HASH);
+        final boolean isAsync = ((null != asyncJob) && asyncJob.booleanValue()) || ((null != remoteAsyncJob) && remoteAsyncJob.booleanValue());
+        final boolean isCacheOnly = (null != cacheOnlyJob) && cacheOnlyJob.booleanValue();
+        final String remoteParams = isMultipartRequest ? getRemoteParams(jobProperties, resultProperties, false, "returntype=binary") : getRemoteParams(jobProperties, resultProperties, true);
+        final boolean logTrace = ClientManager.isLogTrace();
+        final String remoteMethod = jobProperties.containsKey(Properties.PROP_REMOTE_METHOD) ? (String) jobProperties.get(Properties.PROP_REMOTE_METHOD) : "";
+        final String logTitle = (remoteMethod.equals("transformimage")) ?
+            ("ImageServer " + (isAsync ? "asynchronous " : "") + "remote transformation") :
+            ("Documentconverter " + (isAsync ? "asynchronous " : "") + "remote conversion");
 
         if ((null != m_remoteUrl) && (null != remoteParams)) {
             final String callCookie = (String) jobProperties.get(Properties.PROP_CONVERTER_COOKIE);
             URL remoteUrl = null;
 
-            if (ClientManager.isLogTrace()) {
-                final String remoteMethod = jobProperties.containsKey(Properties.PROP_REMOTE_METHOD) ? (String) jobProperties.get(Properties.PROP_REMOTE_METHOD) : null;
-
-                if (null != remoteMethod) {
-                    if (remoteMethod.equals("transformimage")) {
-                        ClientManager.logTrace("ImageServer remote transformation initiated");
-                    } else {
-                        ClientManager.logTrace("Documentconverter remote conversion initiated ", new LogData("method", remoteMethod));
-                    }
-                }
-
+            if (logTrace) {
+                ClientManager.logTrace(logTitle + " initiated", jobProperties, new LogData("method", remoteMethod));
             }
 
             try {
@@ -354,52 +437,58 @@ class ManagedClientJob {
             if ((null != remoteUrl) && isMultipartRequest) {
                 final String inputType = (String) jobProperties.get(Properties.PROP_INPUT_TYPE);
                 final String inputMimeType = (null != inputType) && (inputType.equals("pdf")) ? "application/pdf" : "application/octet-stream";
-                final Serializable serialObject = getInputBuffer(jobProperties);
-                final TransferObject<Serializable> transferObject = new TransferObject<>(
-                    remoteParams,
-                    serialObject,
-                    "ByteArray.bin",
-                    inputMimeType,
-                    callCookie);
+                final Serializable serialObject = getInputBuffer(jobProperties, resultProperties);
 
-                TransferHelper.postFormData(m_clientManager, remoteUrl, m_serverType, transferObject, new TransferResponseProcessor() {
+                if (null != serialObject) {
+                    final TransferObject<Serializable> transferObject = new TransferObject<>(
+                        remoteParams,
+                        serialObject,
+                        "ByteArray.bin",
+                        inputMimeType,
+                        callCookie);
 
-                    @SuppressWarnings("unchecked")
-                    @Override
-                    public void processResponse(int responseCode, InputStream responseInputStream, String resultCookie) {
-                        if (null != responseInputStream) {
-                            try (ObjectInputStream objInputStream = new ObjectInputStream(new BufferedInputStream(responseInputStream))) {
-                                final Object readObject = objInputStream.readObject();
+                    TransferHelper.postFormData(m_clientManager, remoteUrl, m_serverType, transferObject, new TransferResponseProcessor() {
 
-                                if ((null != readObject) && (readObject instanceof HashMap<?, ?>)) {
-                                    resultProperties.clear();
-                                    resultProperties.putAll((HashMap<? extends String, ?>) readObject);
+                        @SuppressWarnings("unchecked")
+                        @Override
+                        public void processResponse(int responseCode, InputStream responseInputStream, String resultCookie) {
+                            try {
+                                // evaluate result only for synchronous calls with valid result stream
+                                if (!isAsync && (null != responseInputStream)) {
+                                    try (ObjectInputStream objInputStream = new ObjectInputStream(new BufferedInputStream(responseInputStream))) {
+                                        final Object readObject = objInputStream.readObject();
 
-                                    if (null != resultCookie) {
-                                        resultProperties.put(Properties.PROP_RESULT_CONVERTER_COOKIE, resultCookie);
-                                    }
+                                        if ((null != readObject) && (readObject instanceof HashMap<?, ?>)) {
+                                            resultProperties.clear();
+                                            resultProperties.putAll((HashMap<? extends String, ?>) readObject);
 
-                                    // check for old style property and convert to result error
-                                    final Boolean passwordProtected = (Boolean) resultProperties.get("PasswordProtected");
+                                            if (null != resultCookie) {
+                                                resultProperties.put(Properties.PROP_RESULT_CONVERTER_COOKIE, resultCookie);
+                                            }
 
-                                    if ((null != passwordProtected) && passwordProtected.booleanValue()) {
-                                        resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.PASSWORD.errorCode()));
+                                            // check for old style property and convert to result error
+                                            final Boolean passwordProtected = (Boolean) resultProperties.get("PasswordProtected");
+
+                                            if ((null != passwordProtected) && passwordProtected.booleanValue()) {
+                                                resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, Integer.valueOf(JobError.PASSWORD.errorCode()));
+                                            }
+                                        }
+                                    } catch (Exception e) {
+                                        resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, CONNECTION_TIMEOUT_ERRORCODE);
+                                        ClientManager.logExcp(e);
                                     }
                                 }
-                            } catch (Exception e) {
-                                resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, CONNECTION_TIMEOUT_ERRORCODE);
-                                ClientManager.logExcp(e);
                             } finally {
                                 IOUtils.closeQuietly(responseInputStream);
                             }
                         }
+                    });
+
+                    final byte[] resultBuffer = (byte[]) resultProperties.get(Properties.PROP_RESULT_BUFFER);
+
+                    if (null != resultBuffer) {
+                        ret = new ByteArrayInputStream(resultBuffer);
                     }
-                });
-
-                final byte[] resultBuffer = (byte[]) resultProperties.get(Properties.PROP_RESULT_BUFFER);
-
-                if (null != resultBuffer) {
-                    ret = new ByteArrayInputStream(resultBuffer);
                 }
             } else {
                 final TransferObject<byte[]> transferObject = new TransferObject<>(remoteParams, callCookie);
@@ -409,58 +498,71 @@ class ManagedClientJob {
 
                     @Override
                     public void processResponse(int responseCode, InputStream responseInputStream, String resultCookie) {
-                        if (null != responseInputStream) {
-                            try (BufferedReader inputReader = new BufferedReader(new InputStreamReader(responseInputStream))) {
-                                for (String readLine = null; (readLine = inputReader.readLine()) != null;) {
-                                    response.append(readLine).append('\n');
-                                }
+                        try {
+                            // evaluate result only for synchronous calls with valid result stream
+                            if (!isAsync && (null != responseInputStream)) {
+                                try (BufferedReader inputReader = new BufferedReader(new InputStreamReader(responseInputStream))) {
+                                    for (String readLine = null; (readLine = inputReader.readLine()) != null;) {
+                                        response.append(readLine).append('\n');
+                                    }
 
-                                if (null != resultCookie) {
-                                    resultProperties.put(Properties.PROP_RESULT_CONVERTER_COOKIE, resultCookie);
+                                    if (null != resultCookie) {
+                                        resultProperties.put(Properties.PROP_RESULT_CONVERTER_COOKIE, resultCookie);
+                                    }
+                                } catch (Exception e) {
+                                    resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, CONNECTION_TIMEOUT_ERRORCODE);
+                                    ClientManager.logExcp(e);
                                 }
-                            } catch (Exception e) {
-                                resultProperties.put(Properties.PROP_RESULT_ERROR_CODE, CONNECTION_TIMEOUT_ERRORCODE);
-                                ClientManager.logExcp(e);
-                            } finally {
-                                IOUtils.closeQuietly(responseInputStream);
                             }
+                        } finally {
+                            IOUtils.closeQuietly(responseInputStream);
                         }
                     }
                 });
 
-                // parse JSON response
-                try {
-                    ret = parseJSONResponse(new JSONObject(new JSONTokener(response.toString())), resultProperties);
-                } catch (JSONException e) {
-                    ClientManager.logExcp(e);
+                // parse JSON response only on case of synchronous jobs,
+                // since we won't get a valid answer for async requests
+                if (!isAsync) {
+                    try {
+                        ret = parseJSONResponse(new JSONObject(new JSONTokener(response.toString())), resultProperties);
+                    } catch (JSONException e) {
+                        ClientManager.logExcp(e);
+                    }
                 }
             }
         }
 
-        if (!resultProperties.containsKey(Properties.PROP_RESULT_ERROR_CODE) || (0 == ((Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE)).intValue())) {
-            if (ClientManager.isLogDebug()) {
-                ClientManager.logDebug("Documentconverter remote conversion finished");
+        if (isAsync) {
+            if (logTrace) {
+                ClientManager.logTrace(logTitle + " implicitly succeeded", jobProperties);
+            }
+        } else if (!resultProperties.containsKey(Properties.PROP_RESULT_ERROR_CODE) || (0 == ((Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE)).intValue())) {
+            if (logTrace) {
+                ClientManager.logTrace(logTitle + " succeeded", jobProperties);
             }
         } else if (ClientManager.isLogWarn()) {
             // don't log warnings for cacheOnly jobs /
             // don't log warnings for async jobs /
             // don't log warnings for remote cachehash jobs
-            final Boolean cacheOnlyJob = (Boolean) jobProperties.get(Properties.PROP_CACHE_ONLY);
-            final Boolean asyncJob = (Boolean) jobProperties.get(Properties.PROP_ASYNC);
-            final String remoteCacheHash = (String) jobProperties.get(Properties.PROP_REMOTE_CACHE_HASH);
-
+            // ------------------------------------------------------
             // all above mentioned types of jobs may return without a
             // valid or with an error result. This should not be logged
             // as an error/warning at all here, since there'll either be a
             // fallback conversion or the caller is not interested in the
             // the current result of these 'try and ignore' conversions
-            if (((null == cacheOnlyJob) || !cacheOnlyJob.booleanValue()) && ((null == asyncJob) || !asyncJob.booleanValue()) && ((null == remoteCacheHash) || remoteCacheHash.isEmpty())) {
-                final Integer resultErrorCode = (Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE);
+            if (!isCacheOnly && !isAsync && StringUtils.isEmpty(remoteCacheHash)) {
                 final StringBuilder errorMessageBuilder = new StringBuilder("Documentconverter received a remote conversion ");
+                Integer resultErrorCode = (Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE);
+
+                if (null == resultErrorCode) {
+                    resultErrorCode = new Integer(JobError.GENERAL.errorCode());
+                }
 
                 ClientManager.logWarn(errorMessageBuilder.append(
-                    ((null != resultErrorCode) && resultErrorCode.equals(CONNECTION_TIMEOUT_ERRORCODE)) ?
-                        "timeout error" : "general error").toString(), jobProperties);
+                    resultErrorCode.equals(CONNECTION_TIMEOUT_ERRORCODE) ? "timeout error" :
+                        (JobError.fromErrorCode(resultErrorCode.intValue()) == JobError.NO_CONTENT) ? "no content error" :
+                            (JobError.fromErrorCode(resultErrorCode.intValue()) == JobError.PASSWORD) ? "password error" :
+                                (JobError.fromErrorCode(resultErrorCode.intValue()) == JobError.MAX_SOURCESIZE) ? "max source size error" : "general error").toString());
             }
         }
 
@@ -472,7 +574,7 @@ class ManagedClientJob {
      * @param jobProperties
      * @return
      */
-    private String getRemoteParams(HashMap<String, Object> jobProperties, boolean includeDataUrl, String... additionalParams) {
+    private String getRemoteParams(final HashMap<String, Object> jobProperties, final HashMap<String, Object> resultProperties, boolean includeDataUrl, String... additionalParams) {
         StringBuilder paramBuilder = new StringBuilder("action=rconvert");
         Object curParam = null;
 
@@ -598,6 +700,16 @@ class ManagedClientJob {
                 paramBuilder.append("&async=").append(((Boolean) curParam).toString());
             }
 
+            // RemoteAsync (treated as async for remote side)
+            if (null != (curParam = jobProperties.get(Properties.PROP_REMOTE_ASYNC))) {
+                paramBuilder.append("&async=").append(((Boolean) curParam).toString());
+            }
+
+            // IsAvailable
+            if (null != (curParam = jobProperties.get(Properties.PROP_QUERY_AVAILABILITY))) {
+                paramBuilder.append("&queryavailability=").append(((Boolean) curParam).toString());
+            }
+
             // ImageFormat
             if (null != (curParam = jobProperties.get(Properties.PROP_IMAGE_FORMAT))) {
                 paramBuilder.append("&imageformat=").append((String) curParam);
@@ -637,7 +749,7 @@ class ManagedClientJob {
 
             if (includeDataUrl) {
                 // add base64 encoded data from InputFile or InputStream as data Url
-                final byte[] inputBuffer = getInputBuffer(jobProperties);
+                final byte[] inputBuffer = getInputBuffer(jobProperties, resultProperties);
 
                 if (null != inputBuffer) {
                     final String inputType = (String) jobProperties.get(Properties.PROP_INPUT_TYPE);
@@ -790,6 +902,8 @@ class ManagedClientJob {
 
     protected static final String RCALL_CONVERT = "convert";
 
+    protected static final String RCALL_QUERY_AVAILABILITY = "queryavailability";
+
     protected static final String RCALL_BEGINPAGECONVERSION = "beginpageconversion";
 
     protected static final String RCALL_GETCONVERSIONPAGE = "getconversionpage";
