@@ -59,7 +59,9 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
+
 import org.apache.commons.io.IOUtils;
+
 import com.openexchange.documentconverter.LogData;
 import com.openexchange.documentconverter.NonNull;
 import com.openexchange.documentconverter.ServerType;
@@ -228,7 +230,7 @@ class RemoteValidator {
      */
     synchronized void trigger() {
         if ((null != m_callable) && m_isRunning.compareAndSet(false, true)) {
-            Executors.newSingleThreadExecutor().submit(new FutureTask<Integer>(m_callable));
+            Executors.newSingleThreadExecutor().submit(new FutureTask<>(m_callable));
         }
     }
 
