@@ -46,11 +46,11 @@
  *
  */
 
-
 package com.openexchange.documentconverter.client.json;
 
 import java.io.InputStream;
 import java.util.Collections;
+import java.util.EnumSet;
 import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
@@ -65,10 +65,8 @@ import com.openexchange.ajax.fileholder.IFileHolder;
 import com.openexchange.ajax.requesthandler.AJAXActionService;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
-import com.openexchange.capabilities.CapabilityService;
-import com.openexchange.capabilities.CapabilitySet;
+import com.openexchange.ajax.requesthandler.crypto.CryptographicServiceAuthenticationFactory;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.documentconverter.Feature;
 import com.openexchange.documentconverter.JobError;
 import com.openexchange.documentconverter.NonNull;
 import com.openexchange.exception.OXException;
@@ -77,6 +75,8 @@ import com.openexchange.file.storage.FileStorageFileAccess;
 import com.openexchange.file.storage.composition.FolderID;
 import com.openexchange.file.storage.composition.IDBasedFileAccess;
 import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
+import com.openexchange.file.storage.composition.crypto.CryptographicAwareIDBasedFileAccessFactory;
+import com.openexchange.file.storage.composition.crypto.CryptographyMode;
 import com.openexchange.filemanagement.ManagedFile;
 import com.openexchange.filemanagement.ManagedFileManagement;
 import com.openexchange.groupware.attach.Attachments;
@@ -86,7 +86,6 @@ import com.openexchange.guard.api.Header;
 import com.openexchange.mail.MailServletInterface;
 import com.openexchange.mail.dataobjects.MailPart;
 import com.openexchange.server.ServiceLookup;
-import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSession;
 
 /**
@@ -96,6 +95,14 @@ import com.openexchange.tools.session.ServerSession;
  */
 public class DocumentConverterAction implements AJAXActionService {
 
+    // - Statics ---------------------------------------------------------------
+
+    // TODO (KA): check list of parameters for completeness prior to every release!!!
+    // The parameter list is used for the fast detection of ManagedFiles, based on
+    // the set of all given attributes to create a unique identifier. The ManagedFile itself
+    // is created for the speedup of chunked requests, using the same source again and again
+    static final private String[] idParamArray = { "uid", "id", "folder_id", "version", "source", "attached", "module", "cryptoAuth" };
+
     /**
      * Initializes a new {@link DocumentConverterAction}.
      *
@@ -140,70 +147,69 @@ public class DocumentConverterAction implements AJAXActionService {
     }
 
     /**
-     * @param session
-     * @return
-     */
-    boolean isFeatureSupportedForUser(Session session, Feature feature) {
-        boolean ret = false;
-
-        if (null != session) {
-            final CapabilityService capService = m_services.getService(CapabilityService.class);
-
-            try {
-                @SuppressWarnings("unused") final CapabilitySet capSet = (null != capService) ? capService.getCapabilities(session) : null;
-
-                /* test prototype
-                if ((null != capSet) && capSet.contains("capabilityTestString")) {
-                    ret = true;
-                }
-                */
-            } catch (final OXException e) {
-                logException(e);
-            }
-        }
-
-        return ret;
-    }
-
-    /**@param documentId
      * @param request
      * @param session
      * @param jobError
      * @return
      */
-    protected ManagedFile getManagedFileForRequestDocument(@NonNull AJAXRequestData request, @NonNull ServerSession session, @NonNull JobError[] jobError) {
+    protected ManagedFile getManagedFileForRequestDocument(@NonNull final AJAXRequestData request, final @NonNull ServerSession session, @NonNull JobError[] jobError) {
         final String documentId = getDocumentId(request);
         ManagedFile ret = null;
 
         try {
-            ret = m_managedFileManagement.contains(documentId) ? m_managedFileManagement.getByID(documentId) : null;
-        } catch (final OXException e) {
-            logException(e);
-        }
-
-        if (null == ret) {
-            final String fileId = request.getParameter("id");
-            final String source = request.getParameter("source");
-            final String attached = request.getParameter("attached");
-            final String module = request.getParameter("module");
-            final String folderId = request.getParameter("folder_id");
-            final String version = request.getParameter("version");
-            final String guardUrl = request.getParameter("guardUrl");
-            final boolean mailCheck = ((attached != null) && (folderId != null) && (attached.length() > 0));
-            final boolean taskCheck = mailCheck && (module != null);
-            MailServletInterface mailInterface = null;
-            InputStream documentStm = null;
+            FileIdLocker.lock(documentId);
 
             try {
-                if ((null != source) && source.equals("mail") && mailCheck) {
-                    mailInterface = MailServletInterface.getInstance(session);
-                    final MailPart mailPart = mailInterface.getMessageAttachment(folderId, fileId, attached, false);
+                ret = m_managedFileManagement.contains(documentId) ? m_managedFileManagement.getByID(documentId) : null;
+            } catch (final OXException e) {
+                logException(e);
+            }
 
-                    if (null != mailPart) {
-                        documentStm = mailPart.getInputStream();
-                    }
-                } else if ((null != source) && (source.equals("tasks") || source.equals("calendar") || source.equals("contacts")) && taskCheck) {
-                    documentStm = Attachments.getInstance().getAttachedFile(
+            if (null == ret) {
+                final String fileId = request.getParameter("id");
+                final String source = request.getParameter("source");
+                final String attached = request.getParameter("attached");
+                final String module = request.getParameter("module");
+                final String folderId = request.getParameter("folder_id");
+                final String version = request.getParameter("version");
+                final String cryptoAuth = request.getParameter("cryptoAuth");
+                final String guardUrl = request.getParameter("guardUrl");
+                final boolean mailCheck = ((attached != null) && (folderId != null) && (attached.length() > 0));
+                final boolean taskCheck = mailCheck && (module != null);
+                final boolean oldStyleGuardCheck = (null != guardUrl) && (guardUrl.length() > 10);
+                @SuppressWarnings("resource") MailServletInterface mailServletInterface = null; // closing done in finally block
+                @SuppressWarnings("resource") InputStream documentStm = null; // closing done in finally block
+
+                try {
+                    if (mailCheck && "mail".equals(source)) {
+                        // mail attachment
+
+                        if (StringUtils.isNotEmpty(cryptoAuth)) {
+                            // If GuardMail, pull authentication from request
+                            final CryptographicServiceAuthenticationFactory encryptionAuthenticationFactory = m_services.getOptionalService(CryptographicServiceAuthenticationFactory.class);
+                            String authentication = null;
+
+                            if (null != encryptionAuthenticationFactory) {
+                                authentication = encryptionAuthenticationFactory.createAuthenticationFrom(request.optHttpServletRequest());
+                            }
+
+                            // not yet available in 7.8.3, use default instance
+                            // mailServletInterface = MailServletInterface.getInstanceWithDecryptionSupport(session,  authentication);
+                            mailServletInterface = MailServletInterface.getInstance(session);
+                        } else {
+                            mailServletInterface = MailServletInterface.getInstance(session);
+                        }
+
+                        if (null != mailServletInterface) {
+                            final MailPart mailPart = mailServletInterface.getMessageAttachment(folderId, fileId, attached, false);
+
+                            if (null != mailPart) {
+                                documentStm = mailPart.getInputStream();
+                            }
+                        }
+                    } else if (taskCheck && ("tasks".equals(source) || "calendar".equals(source) || "contacts".equals(source))) {
+                        // other attachment from tasks, calendar or contact
+                        documentStm = Attachments.getInstance().getAttachedFile(
                             session,
                             Integer.parseInt(folderId),
                             Integer.parseInt(attached),
@@ -212,36 +218,61 @@ public class DocumentConverterAction implements AJAXActionService {
                             session.getContext(),
                             session.getUser(),
                             session.getUserConfiguration());
-                } else if ((null != source) && (source.equals("guard")) && (null != guardUrl) && (guardUrl.length() > 0)) {
-                    final GuardApi guardApi = m_services.getOptionalService(GuardApi.class);
+                    } else if ((null != source) && (source.equals("guard")) && oldStyleGuardCheck) {
+                        // Old style Guard encryption may still occur. This is used for email attachments,
+                        // The guardUrl paameter provides information to retrieve data from email mime body
+                        final GuardApi guardApi = m_services.getOptionalService(GuardApi.class);
 
-                    if (null != guardApi) {
-                        final HttpServletRequest httpServletRequest = request.optHttpServletRequest();
+                        if (null != guardApi) {
+                            final HttpServletRequest httpServletRequest = request.optHttpServletRequest();
 
-                        if (null != httpServletRequest) {
-                            final String userAgent = httpServletRequest.getHeader("User-Agent");
+                            if (null != httpServletRequest) {
+                                final String userAgent = httpServletRequest.getHeader("User-Agent");
 
-                            documentStm = guardApi.requestSessionSensitiveResource(
+                                documentStm = guardApi.requestSessionSensitiveResource(
                                     parseQueryStringFromUrl(guardUrl),
-                                    session, GuardApis.extractCookiesFrom(httpServletRequest, session),
-                                    (null == userAgent) ?
-                                            Collections.<Header> emptyList() :
-                                                Collections.singletonList(new Header("User-Agent", userAgent)));
+                                    session,
+                                    GuardApis.extractCookiesFrom(httpServletRequest, session),
+                                    (null == userAgent) ? Collections.<Header> emptyList() : Collections.singletonList(new Header("User-Agent", userAgent)));
+                            }
                         }
-                    }
-                } else {
-                    final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
-                    final IDBasedFileAccess fileAccess = (null != fileFactory) ? fileFactory.createAccess(session) : null;
+                    } else {
+                        // use FileStore as default data source
+                        final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
+                        IDBasedFileAccess fileAccess = (null != fileFactory) ? fileFactory.createAccess(session) : null;
 
-                    if (null != fileAccess) {
-                        String versionToUse = version;
+                        if ((null != fileAccess) && "guard".equals(source)) {
+                            if (StringUtils.isNotEmpty(cryptoAuth)) {
+                                // If GuardMail, get authentication
+                                final CryptographicAwareIDBasedFileAccessFactory encryptionAwareFileAccessFactory = m_services.getOptionalService(CryptographicAwareIDBasedFileAccessFactory.class);
+                                final CryptographicServiceAuthenticationFactory encryptionAuthenticationFactory = m_services.getOptionalService(CryptographicServiceAuthenticationFactory.class);
 
-                        // #46555 map version String("null") to null
-                        if ((null != versionToUse) && versionToUse.equalsIgnoreCase("null")) {
-                            versionToUse = FileStorageFileAccess.CURRENT_VERSION;
+                                if ((null != encryptionAwareFileAccessFactory) && (null != encryptionAuthenticationFactory)) {
+                                    final EnumSet<CryptographyMode> cryptMode = EnumSet.of(CryptographyMode.DECRYPT);
+
+                                    fileAccess = encryptionAwareFileAccessFactory.createAccess(fileAccess, cryptMode, session,
+                                        encryptionAuthenticationFactory.createAuthenticationFrom(request.optHttpServletRequest()));
+                                } else {
+                                    // file is encrypted, but we can't decrypt...
+                                    fileAccess = null;
+                                }
+                            } else {
+                                fileAccess = null;
+                            }
+
+                            // set output paramter jobError to JobError.PASSWORD in
+                            // case we got no valid fileAccess for given Guard parameters
+                            if ((null == fileAccess) && (jobError.length > 0)) {
+                                jobError[0] = JobError.PASSWORD;
+                            }
                         }
 
-                        try {
+                        if (null != fileAccess) {
+                            // #46555 map version String("null") to null
+                            String versionToUse = "null".equalsIgnoreCase(version) ?
+                                FileStorageFileAccess.CURRENT_VERSION :
+                                    version;
+
                             // #46555 check for versioning support
                             if (FileStorageFileAccess.CURRENT_VERSION != versionToUse) {
                                 final FolderID folderIdentifier = new FolderID(folderId);
@@ -250,29 +281,23 @@ public class DocumentConverterAction implements AJAXActionService {
                                     versionToUse = FileStorageFileAccess.CURRENT_VERSION;
                                 }
                             }
-                        } catch (Exception e) {
-                            logException(e);
+
+                            documentStm = fileAccess.getDocument(fileId, versionToUse);
                         }
-
-                        documentStm = fileAccess.getDocument(fileId, versionToUse);
                     }
-                }
-            } catch (final OXException e) {
-                logException(e);
-            }
 
-            if (null != documentStm) {
-                try {
-                    ret = m_managedFileManagement.createManagedFile(documentId, documentStm);
-                } catch (final OXException e) {
+                    if (null != documentStm) {
+                        ret = m_managedFileManagement.createManagedFile(documentId, documentStm);
+                    }
+                } catch (final Exception e) {
                     logException(e);
                 } finally {
                     IOUtils.closeQuietly(documentStm);
-                    documentStm = null;
+                    IOUtils.closeQuietly(mailServletInterface);
                 }
             }
-
-            IOUtils.closeQuietly(mailInterface);
+        } finally {
+            FileIdLocker.unlock(documentId);
         }
 
         return ret;
@@ -296,36 +321,36 @@ public class DocumentConverterAction implements AJAXActionService {
                 String errorCause = null;
 
                 switch (jobError) {
-                case NONE: {
-                    break;
-                }
+                    case NONE: {
+                        break;
+                    }
 
-                case TIMEOUT: {
-                    errorCause = "timeout";
-                    break;
-                }
+                    case TIMEOUT: {
+                        errorCause = "timeout";
+                        break;
+                    }
 
-                case PASSWORD: {
-                    errorCause = "passwordProtected";
-                    break;
-                }
+                    case PASSWORD: {
+                        errorCause = "passwordProtected";
+                        break;
+                    }
 
-                case NO_CONTENT: {
-                    errorCause = "noContent";
-                    break;
-                }
+                    case NO_CONTENT: {
+                        errorCause = "noContent";
+                        break;
+                    }
 
-                case MAX_SOURCESIZE: {
-                    errorCause = "maxSourceSizeExceeded";
-                    jsonResult.put("MaxSourceSizeMB", "" + m_maxSourceFileSizeMB);
-                    break;
-                }
+                    case MAX_SOURCESIZE: {
+                        errorCause = "maxSourceSizeExceeded";
+                        jsonResult.put("MaxSourceSizeMB", "" + m_maxSourceFileSizeMB);
+                        break;
+                    }
 
-                case GENERAL:
-                default: {
-                    errorCause = "filterError";
-                    break;
-                }
+                    case GENERAL:
+                    default: {
+                        errorCause = "filterError";
+                        break;
+                    }
                 }
 
                 if (null != errorCause) {
@@ -343,12 +368,11 @@ public class DocumentConverterAction implements AJAXActionService {
     }
 
     /**
-     * Parses a query string from given URL.
-     *
-     * @param uri The URI string to be parsed
-     * @return The parsed parameters or <code>null</code>
-     */
-
+    * Parses a query string from given URL.
+    *
+    * @param uri The URI string to be parsed
+    * @return The parsed parameters or <code>null</code>
+    */
     Map<String, String> parseQueryStringFromUrl(String uri) {
         if (StringUtils.isEmpty(uri)) {
             return null;
@@ -377,7 +401,7 @@ public class DocumentConverterAction implements AJAXActionService {
      * @param queryStr The query string to be parsed
      * @return The parsed parameters
      */
-    Map<String, String> parseQueryString(String queryStr) {
+    static Map<String, String> parseQueryString(String queryStr) {
         final String[] pairs = PATTERN_SPLIT.split(queryStr, 0);
         final Map<String, String> map = new LinkedHashMap<>(4);
         for (String pair : pairs) {
@@ -410,7 +434,7 @@ public class DocumentConverterAction implements AJAXActionService {
      * @return
      */
     static protected String getDocumentId(@NonNull AJAXRequestData request, String... additionalData) {
-        final StringBuilder idBuilder = new StringBuilder();
+        final StringBuilder idBuilder = new StringBuilder(128);
         final String[][] allParamArrays = { idParamArray, additionalData };
 
         for (final String[] curParamArray : allParamArrays) {
@@ -422,25 +446,13 @@ public class DocumentConverterAction implements AJAXActionService {
 
                     if (isParam) {
                         curValue = request.getParameter(curParam);
-
-                        // check for non-unique CURRENT_VERSION file version
-                        // and create unique version, if necessary
-                        if (curParam.equalsIgnoreCase("version") && (FileStorageFileAccess.CURRENT_VERSION == curValue)) {
-                            curValue = Long.toString(m_uniqueVersion.decrementAndGet());
-                        }
-
                     }
 
-                    idBuilder.append(StringUtils.isEmpty(curValue) ? "null" : curValue).append('_');
+                    idBuilder.append(curValue).append('#');
                 }
             }
         }
 
-        // remove last dash
-        if (idBuilder.length() > 1) {
-            idBuilder.setLength(idBuilder.length() - 1);
-        }
-
         return idBuilder.toString();
     }
 
@@ -450,23 +462,12 @@ public class DocumentConverterAction implements AJAXActionService {
      * @return
      */
     static protected String getReplacedExtensionFileName(String fileName, @NonNull String newExtension) {
-        final int dotPos = StringUtils.isEmpty(fileName) ? -2 : fileName.lastIndexOf('.');
-        StringBuffer newFileNameBuffer = new StringBuffer((null != fileName) ? (fileName.length() + 8) : 16);
+        final String usedFileName = StringUtils.isEmpty(fileName) ? "unknown" : fileName;
+        final int dotPos = usedFileName.lastIndexOf('.');
 
-        if (dotPos > -1) {
-            newFileNameBuffer.append(fileName.substring(0, dotPos + 1)).append(newExtension);
-        } else {
-            newFileNameBuffer.append(StringUtils.isEmpty(fileName) ? "unknown" : fileName).append('.').append(newExtension);
-        }
-
-        return newFileNameBuffer.toString();
+        return new StringBuffer((dotPos > -1) ? usedFileName.substring(0, dotPos + 1) : usedFileName).append(newExtension).toString();
     }
 
-
-    // - Statics ---------------------------------------------------------------
-
-    static final private String[] idParamArray = { "uid", "id", "folder_id", "version", "source", "attached", "module" };
-
     // - Members ---------------------------------------------------------------
 
     static protected AtomicLong m_uniqueVersion = new AtomicLong(0);
@@ -477,4 +478,3 @@ public class DocumentConverterAction implements AJAXActionService {
 
     protected long m_maxSourceFileSizeMB = 32;
 }
-
