@@ -56,7 +56,10 @@ import java.util.HashMap;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.StringUtils;
+import org.json.JSONException;
 import org.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import com.openexchange.ajax.container.FileHolder;
 import com.openexchange.ajax.fileholder.IFileHolder;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
@@ -110,6 +113,11 @@ import com.openexchange.tools.session.ServerSession;
 @DispatcherNotes(defaultFormat = "file", allowPublicSession = true)
 public class GetDocumentAction extends DocumentConverterAction {
 
+    final private static String KEY_ERRORCODE = "errorCode";
+
+    final private static int ERRORCODE_NONE = 0;
+    final private static int ERRORCODE_BAD_REQUEST = 400;
+
     /**
      * Initializes a new {@link GetDocumentAction}.
      *
@@ -133,6 +141,8 @@ public class GetDocumentAction extends DocumentConverterAction {
         final boolean convertToPDF = targetFormat.equals("pdf") && ((null == mimeType) || !mimeType.equalsIgnoreCase("application/pdf"));
         String saveAsFileName = request.getParameter("saveas_filename");
         String saveAsFolderId = request.getParameter("saveas_folder_id");
+        final String async = request.getParameter("async");
+        final boolean isAsync = (null != async) && async.equalsIgnoreCase("true");
         final JobError[] jobError = { JobError.NONE };
         ManagedFile managedResultFile = null;
         JSONObject jsonResult = null;
@@ -141,8 +151,6 @@ public class GetDocumentAction extends DocumentConverterAction {
         if (convertToPDF && (null != docConverter)) {
             final User sessionUser = session.getUser();
             String prefLanguage = null;
-            Integer featuresId = null;
-            InputStream pdfResultStm = null;
             ManagedFile managedSourceFile = null;
 
             // user language
@@ -154,13 +162,13 @@ public class GetDocumentAction extends DocumentConverterAction {
                 }
             }
 
-            final String resultDocumentId = getDocumentId(request, "pdf", prefLanguage, (null  != featuresId) ? featuresId.toString() : null);
+            final String resultDocumentId = getDocumentId(request, "pdf", prefLanguage);
 
             try {
+                FileIdLocker.lock(resultDocumentId);
                 managedResultFile = m_managedFileManagement.contains(resultDocumentId) ? m_managedFileManagement.getByID(resultDocumentId) : null;
 
                 if (null == managedResultFile) {
-                    final String async = request.getParameter("async");
                     String priority = request.getParameter("priority");
                     priority = (null != priority) ? priority.toLowerCase() : "";
                     final JobPriority jobPriority = priority.equals("background") ? JobPriority.BACKGROUND
@@ -180,18 +188,25 @@ public class GetDocumentAction extends DocumentConverterAction {
                             jobProperties.put(Properties.PROP_INFO_FILENAME, fileName);
                         }
 
-                        if ((null != async) && async.equalsIgnoreCase("true")) {
+                        if (isAsync) {
                             jobProperties.put(Properties.PROP_ASYNC, Boolean.TRUE);
+                            jobProperties.put(Properties.PROP_PRIORITY, JobPriority.BACKGROUND);
                         }
 
                         // checked warning => stream wil either be directly closed or indirectly after write of the fileHolder
-                        if (null != (pdfResultStm = docConverter.convert("pdf", jobProperties, resultProperties))) {
-                            fileName = getReplacedExtensionFileName(fileName, "pdf");
-                            mimeType = "application/pdf";
-                            managedResultFile = m_managedFileManagement.createManagedFile(resultDocumentId, pdfResultStm);
-                        } else {
-                            final Integer errorCode = (Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE);
-                            jobError[0] = (null != errorCode) ? JobError.fromErrorCode(errorCode.intValue()) : JobError.GENERAL;
+                        try (InputStream pdfResultStm = docConverter.convert("pdf", jobProperties, resultProperties)) {
+                            if (null != pdfResultStm) {
+                                fileName = getReplacedExtensionFileName(fileName, "pdf");
+                                mimeType = "application/pdf";
+                                managedResultFile = m_managedFileManagement.createManagedFile(resultDocumentId, pdfResultStm);
+                            } else {
+                                final Integer errorCode = (Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE);
+                                jobError[0] = isAsync ?
+                                    JobError.NONE :
+                                        ((null != errorCode) ?
+                                            JobError.fromErrorCode(errorCode.intValue()) :
+                                                JobError.GENERAL);
+                            }
                         }
                     }
                 } else {
@@ -201,14 +216,9 @@ public class GetDocumentAction extends DocumentConverterAction {
             } catch (Exception e) {
                 logException(e);
             } finally {
-                org.apache.commons.io.IOUtils.closeQuietly(pdfResultStm);
-
-                if (null != managedSourceFile) {
-                    managedSourceFile.delete();
-                    managedSourceFile = null;
-                }
+                FileIdLocker.unlock(resultDocumentId);
             }
-        } else {
+        } else if (!isAsync) {
             // In case of PDF input and output, the result file is equal to the source file
             managedResultFile = getManagedFileForRequestDocument(request, session, jobError);
 
@@ -216,7 +226,6 @@ public class GetDocumentAction extends DocumentConverterAction {
             // do an encryption test to get a valid error code in
             // case of an encrypted PDF
             if ((null != managedResultFile) && (null != encTest) && encTest.equalsIgnoreCase("pdf") && (null != docConverter)) {
-                final String async = request.getParameter("async");
                 String priority = request.getParameter("priority");
 
                 priority = (null != priority) ? priority.toLowerCase() : "";
@@ -234,28 +243,35 @@ public class GetDocumentAction extends DocumentConverterAction {
                     jobProperties.put(Properties.PROP_INFO_FILENAME, fileName);
                 }
 
-                if ((null != async) && async.equalsIgnoreCase("true")) {
-                    jobProperties.put(Properties.PROP_ASYNC, Boolean.TRUE);
-                }
-
                 // do the PDF encryption test and close possible result stream immediately
                 IOUtils.closeQuietly(docConverter.convert("pdfenctest", jobProperties, resultProperties));
 
                 final Integer errorCode = (Integer) resultProperties.get(Properties.PROP_RESULT_ERROR_CODE);
                 jobError[0] = (null != errorCode) ? JobError.fromErrorCode(errorCode.intValue()) : JobError.GENERAL;
             }
+        } else {
+            // async case for PDF documents
+            jobError[0] = JobError.NONE;
         }
 
         // set the temp file of the managed file as result content
-        if (null != managedResultFile) {
-            final File tempFile = managedResultFile.getFile();
+        try {
+            if (null != managedResultFile) {
+                final File tempFile = managedResultFile.getFile();
+                final String sessionId = request.getParameter("session");
 
-            if (null == (jsonResult = getJSONResultForSaveAs(session, tempFile, saveAsFileName, saveAsFolderId, fileName, mimeType))) {
-                resultFileHolder = new FileHolder(FileHolder.newClosureFor(tempFile),
-                                    tempFile.length(),
-                                    ((null != mimeType) && mimeType.equals("application/pdf")) ? mimeType : "application/octet-stream",
-                                    fileName);
+                jsonResult = getJSONResultForSaveAs(session, sessionId, tempFile, saveAsFileName, saveAsFolderId, fileName, mimeType);
+
+                if (null == jsonResult) {
+                    resultFileHolder = new FileHolder(FileHolder.newClosureFor(tempFile), tempFile.length(), mimeType, fileName);
+                } else if (jsonResult.has(KEY_ERRORCODE) && jsonResult.getInt(KEY_ERRORCODE) > ERRORCODE_NONE) {
+                    return (new AJAXRequestResult().setHttpStatusCode(ERRORCODE_BAD_REQUEST));
+                }
+            } else if (isAsync) {
+                (jsonResult = new JSONObject()).put("async", true);
             }
+        } catch (JSONException e) {
+            logException(e);
         }
 
         return getRequestResult(request, resultFileHolder, jsonResult, jobError[0]);
@@ -270,45 +286,57 @@ public class GetDocumentAction extends DocumentConverterAction {
      * @param givenMimeType
      * @return
      */
-    protected JSONObject getJSONResultForSaveAs(@NonNull ServerSession session, File resultFile, String saveAsFileName, String saveAsFolderId,
+    protected JSONObject getJSONResultForSaveAs(@NonNull ServerSession session, String sessionId, File resultFile, String saveAsFileName, String saveAsFolderId,
                                                 String givenFileName, String givenMimeType) {
         JSONObject ret = null;
 
         if ((null != resultFile) && resultFile.canRead() && (resultFile.length() > 0) && !StringUtils.isEmpty(saveAsFileName) && !StringUtils.isEmpty(saveAsFolderId)) {
-            final IDBasedFolderAccessFactory folderFactory = m_services.getService(IDBasedFolderAccessFactory.class);
-            final IDBasedFolderAccess folderAccess = (null != folderFactory) ? folderFactory.createAccess(session) : null;
+            // #60025: we need to check for an illegal CSRF attempt by checking the
+            // correct sessionId; set errorcode to 400 in case of an invalid request
+            try {
+                if ((null == sessionId) || !sessionId.equals(session.getSessionID())) {
+                    final Logger LOG = LoggerFactory.getLogger(this.getClass());
 
-            if (null != folderAccess) {
-                try {
-                    final FileStorageFolder folder = folderAccess.getFolder(saveAsFolderId);
-                    final FileStoragePermission permission = (null != folder) ? folder.getOwnPermission() : null;
+                    if (null != LOG) {
+                        LOG.error("DC Ajax handler was called with invalid sessionId for save request: " + sessionId);
+                    }
 
-                    if ((null != permission) && (permission.getFolderPermission() >= FileStoragePermission.CREATE_OBJECTS_IN_FOLDER)) {
-                        final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
-                        final IDBasedFileAccess fileAccess = (null != fileFactory) ? fileFactory.createAccess(session) : null;
+                    ret = (new JSONObject()).put(KEY_ERRORCODE, ERRORCODE_BAD_REQUEST);
+                } else {
+                    final IDBasedFolderAccessFactory folderFactory = m_services.getService(IDBasedFolderAccessFactory.class);
+                    final IDBasedFolderAccess folderAccess = (null != folderFactory) ? folderFactory.createAccess(session) : null;
 
-                        if (null != fileAccess) {
-                            final com.openexchange.file.storage.File saveAsFileAccessFile = new DefaultFile();
+                    if (null != folderAccess) {
+                        final FileStorageFolder folder = folderAccess.getFolder(saveAsFolderId);
+                        final FileStoragePermission permission = (null != folder) ? folder.getOwnPermission() : null;
 
-                            saveAsFileAccessFile.setId(FileStorageFileAccess.NEW);
-                            saveAsFileAccessFile.setFolderId(saveAsFolderId);
-                            saveAsFileAccessFile.setFileName(saveAsFileName);
-                            saveAsFileAccessFile.setFileMIMEType(givenMimeType);
+                        if ((null != permission) && (permission.getFolderPermission() >= FileStoragePermission.CREATE_OBJECTS_IN_FOLDER)) {
+                            final IDBasedFileAccessFactory fileFactory = m_services.getService(IDBasedFileAccessFactory.class);
+                            final IDBasedFileAccess fileAccess = (null != fileFactory) ? fileFactory.createAccess(session) : null;
 
-                            try (final InputStream resultFileInputStm = FileUtils.openInputStream(resultFile)) {
-                                String resultFileId = fileAccess.saveDocument(saveAsFileAccessFile, resultFileInputStm, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
+                            if (null != fileAccess) {
+                                final com.openexchange.file.storage.File saveAsFileAccessFile = new DefaultFile();
 
-                                if (!StringUtils.isEmpty(resultFileId)) {
-                                    (ret = new JSONObject()).put("id", resultFileId).put("filename", saveAsFileAccessFile.getFileName());
+                                saveAsFileAccessFile.setId(FileStorageFileAccess.NEW);
+                                saveAsFileAccessFile.setFolderId(saveAsFolderId);
+                                saveAsFileAccessFile.setFileName(saveAsFileName);
+                                saveAsFileAccessFile.setFileMIMEType(givenMimeType);
+
+                                try (final InputStream resultFileInputStm = FileUtils.openInputStream(resultFile)) {
+                                    String resultFileId = fileAccess.saveDocument(saveAsFileAccessFile, resultFileInputStm, FileStorageFileAccess.DISTANT_FUTURE, new ArrayList<Field>());
+
+                                    if (!StringUtils.isEmpty(resultFileId)) {
+                                        (ret = new JSONObject()).put("id", resultFileId).put("filename", saveAsFileAccessFile.getFileName()).put(KEY_ERRORCODE, 0);
+                                    }
                                 }
-                            }
 
-                            fileAccess.commit();
+                                fileAccess.commit();
+                            }
                         }
                     }
-                } catch(Exception e) {
-                    logException(e);
                 }
+            } catch(Exception e) {
+                logException(e);
             }
         }
 
