@@ -71,8 +71,6 @@ import org.apache.poi.poifs.filesystem.DocumentEntry;
 import org.apache.poi.poifs.filesystem.DocumentInputStream;
 import org.apache.poi.poifs.filesystem.POIFSFileSystem;
 import org.apache.poi.util.IOUtils;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 import com.openexchange.conversion.Data;
 import com.openexchange.conversion.DataProperties;
 import com.openexchange.documentconverter.IDocumentConverter;
@@ -170,14 +168,14 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
                         }
 
                         // perform an async request for PDF conversion
-                        if (m_log.isTraceEnabled()) {
-                            m_log.trace("OfficePreviewService performs async PDF conversion for document: " + ((null != fileName) ? fileName : ""));
+                        if (ClientManager.isLogTrace()) {
+                            ClientManager.logTrace("OfficePreviewService performs asynchronous PDF conversion for document: " + ((null != fileName) ? fileName : ""));
                         }
 
                         docConverter.convert("pdf", jobProperties, resultProperties);
                     } else {
-                        if (m_log.isTraceEnabled()) {
-                            m_log.trace("OfficePreviewService received async PDF conversion request for PDF document => no need to convert: " + ((null != fileName) ? fileName : ""));
+                        if (ClientManager.isLogTrace()) {
+                            ClientManager.logTrace("OfficePreviewService received asynchronous PDF conversion request for PDF document => no need to convert: " + ((null != fileName) ? fileName : ""));
                         }
                     }
 
@@ -404,7 +402,7 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
                     metaData.put("content-type", "image/jpeg");
                     metaData.put("resourcename", "image.jpg");
 
-                    m_log.trace("OfficePreviewService fulfills thumbnail request from thumbnail contained in document: " + ((null != fileName) ? fileName : ""));
+                    ClientManager.logTrace("OfficePreviewService fulfills thumbnail request from thumbnail contained in document: " + ((null != fileName) ? fileName : ""));
                     previewDocument = new DocumentConverterPreviewDocument(metaData, null, resultBuffer, Boolean.FALSE);
                 }
             }
@@ -422,7 +420,7 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
                 resultProperties.clear();
 
                 try {
-                    m_log.trace("OfficePreviewService converts document to create thumbnail: " + ((null != fileName) ? fileName : ""));
+                    ClientManager.logTrace("OfficePreviewService converts document to create thumbnail: " + ((null != fileName) ? fileName : ""));
 
                     if (null != (resultInputStm = docConverter.convert("preview", jobProperties, resultProperties))) {
                         // try to get direct access to the result buffer to avoid an array copy;
@@ -686,13 +684,13 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
     final protected static void handleExcp(Exception e) throws OXException {
         if (null != e) {
             if (e instanceof org.apache.poi.poifs.filesystem.OfficeXmlFileException) {
-                m_log.info("Document ambiguity: cannot read binary document format with XML extension.");
+                ClientManager.logInfo("Document ambiguity: cannot read binary document format with XML extension.");
             } else if (e instanceof IOException) {
                 throw PreviewExceptionCodes.IO_ERROR.create(e, e.getMessage());
             } else if (e instanceof RuntimeException) {
                 throw PreviewExceptionCodes.ERROR.create(e, e.getMessage());
             } else {
-                m_log.error(e.getMessage());
+                ClientManager.logExcp(e);
             }
         }
     }
@@ -743,7 +741,7 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
                     }
                 }
             } catch (IOException e) {
-                m_log.error(e.getMessage());
+                ClientManager.logExcp(e);
             } finally {
                 IOUtils.closeQuietly(inputStm);
             }
@@ -966,8 +964,4 @@ public class DocumentConverterPreviewService implements RemoteInternalPreviewSer
 
         return previewLanguage;
     }
-
-    // - Members ---------------------------------------------------------------
-
-    private static Logger m_log = LoggerFactory.getLogger(DocumentConverterPreviewService.class);
 }
